package org.apache.commons.lang3;

import java.time.Duration;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.lang3.time.DurationUtils;

public class ThreadUtils {
   public static final AlwaysTruePredicate ALWAYS_TRUE_PREDICATE = new AlwaysTruePredicate();

   public static Thread findThreadById(long threadId) {
      Collection<Thread> result = findThreads(new ThreadIdPredicate(threadId));
      return result.isEmpty() ? null : (Thread)result.iterator().next();
   }

   public static Thread findThreadById(long threadId, String threadGroupName) {
      Validate.notNull(threadGroupName, "threadGroupName");
      Thread thread = findThreadById(threadId);
      return thread != null && thread.getThreadGroup() != null && thread.getThreadGroup().getName().equals(threadGroupName) ? thread : null;
   }

   public static Thread findThreadById(long threadId, ThreadGroup threadGroup) {
      Validate.notNull(threadGroup, "threadGroup");
      Thread thread = findThreadById(threadId);
      return thread != null && threadGroup.equals(thread.getThreadGroup()) ? thread : null;
   }

   public static Collection<ThreadGroup> findThreadGroups(ThreadGroup group, boolean recurse, ThreadGroupPredicate predicate) {
      Validate.notNull(group, "group");
      Validate.notNull(predicate, "predicate");
      int count = group.activeGroupCount();

      ThreadGroup[] threadGroups;
      do {
         threadGroups = new ThreadGroup[count + count / 2 + 1];
         count = group.enumerate(threadGroups, recurse);
      } while(count >= threadGroups.length);

      List<ThreadGroup> result = new ArrayList(count);

      for(int i = 0; i < count; ++i) {
         if (predicate.test(threadGroups[i])) {
            result.add(threadGroups[i]);
         }
      }

      return Collections.unmodifiableCollection(result);
   }

   public static Collection<ThreadGroup> findThreadGroups(ThreadGroupPredicate predicate) {
      return findThreadGroups(getSystemThreadGroup(), true, predicate);
   }

   public static Collection<ThreadGroup> findThreadGroupsByName(String threadGroupName) {
      return findThreadGroups(new NamePredicate(threadGroupName));
   }

   public static Collection<Thread> findThreads(ThreadGroup group, boolean recurse, ThreadPredicate predicate) {
      Validate.notNull(group, "The group must not be null");
      Validate.notNull(predicate, "The predicate must not be null");
      int count = group.activeCount();

      Thread[] threads;
      do {
         threads = new Thread[count + count / 2 + 1];
         count = group.enumerate(threads, recurse);
      } while(count >= threads.length);

      List<Thread> result = new ArrayList(count);

      for(int i = 0; i < count; ++i) {
         if (predicate.test(threads[i])) {
            result.add(threads[i]);
         }
      }

      return Collections.unmodifiableCollection(result);
   }

   public static Collection<Thread> findThreads(ThreadPredicate predicate) {
      return findThreads(getSystemThreadGroup(), true, predicate);
   }

   public static Collection<Thread> findThreadsByName(String threadName) {
      return findThreads(new NamePredicate(threadName));
   }

   public static Collection<Thread> findThreadsByName(String threadName, String threadGroupName) {
      Validate.notNull(threadName, "threadName");
      Validate.notNull(threadGroupName, "threadGroupName");
      Collection<ThreadGroup> threadGroups = findThreadGroups(new NamePredicate(threadGroupName));
      if (threadGroups.isEmpty()) {
         return Collections.emptyList();
      } else {
         Collection<Thread> result = new ArrayList();
         NamePredicate threadNamePredicate = new NamePredicate(threadName);
         Iterator var5 = threadGroups.iterator();

         while(var5.hasNext()) {
            ThreadGroup group = (ThreadGroup)var5.next();
            result.addAll(findThreads(group, false, threadNamePredicate));
         }

         return Collections.unmodifiableCollection(result);
      }
   }

   public static Collection<Thread> findThreadsByName(String threadName, ThreadGroup threadGroup) {
      return findThreads(threadGroup, false, new NamePredicate(threadName));
   }

   public static Collection<ThreadGroup> getAllThreadGroups() {
      return findThreadGroups(ALWAYS_TRUE_PREDICATE);
   }

   public static Collection<Thread> getAllThreads() {
      return findThreads(ALWAYS_TRUE_PREDICATE);
   }

   public static ThreadGroup getSystemThreadGroup() {
      ThreadGroup threadGroup;
      for(threadGroup = Thread.currentThread().getThreadGroup(); threadGroup.getParent() != null; threadGroup = threadGroup.getParent()) {
      }

      return threadGroup;
   }

   public static void join(Thread thread, Duration duration) throws InterruptedException {
      DurationUtils.accept(thread::join, duration);
   }

   public static void sleep(Duration duration) throws InterruptedException {
      DurationUtils.accept(Thread::sleep, duration);
   }

   @FunctionalInterface
   public interface ThreadPredicate {
      boolean test(Thread var1);
   }

   public static class ThreadIdPredicate implements ThreadPredicate {
      private final long threadId;

      public ThreadIdPredicate(long threadId) {
         if (threadId <= 0L) {
            throw new IllegalArgumentException("The thread id must be greater than zero");
         } else {
            this.threadId = threadId;
         }
      }

      public boolean test(Thread thread) {
         return thread != null && thread.getId() == this.threadId;
      }
   }

   @FunctionalInterface
   public interface ThreadGroupPredicate {
      boolean test(ThreadGroup var1);
   }

   public static class NamePredicate implements ThreadPredicate, ThreadGroupPredicate {
      private final String name;

      public NamePredicate(String name) {
         Validate.notNull(name, "name");
         this.name = name;
      }

      public boolean test(Thread thread) {
         return thread != null && thread.getName().equals(this.name);
      }

      public boolean test(ThreadGroup threadGroup) {
         return threadGroup != null && threadGroup.getName().equals(this.name);
      }
   }

   private static final class AlwaysTruePredicate implements ThreadPredicate, ThreadGroupPredicate {
      private AlwaysTruePredicate() {
      }

      public boolean test(Thread thread) {
         return true;
      }

      public boolean test(ThreadGroup threadGroup) {
         return true;
      }

      // $FF: synthetic method
      AlwaysTruePredicate(Object x0) {
         this();
      }
   }
}
